#
# Copyright 2019 David Carter <dcarter@davesrocketshop.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"), 
# to deal in the Software without restriction, including without limitation 
# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
# and/or sell copies of the Software, and to permit persons to whom the 
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in 
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
# IN THE SOFTWARE.

#Author-David Carter

import adsk.core, adsk.fusion, adsk.cam
import math
from .NoseShapeHandler import NoseShapeHandler

    
class EllipseShapeHandler(NoseShapeHandler):
            
    def ellipseRadius(self, length, radius, x):
        # Calulate the radius at a point x along the major axis
        y = math.sqrt(radius*radius * (1.0 - ((x*x)/(length*length))))
        return y
    
    def drawSolid(self, sketch):
        center = adsk.core.Point3D.create(0,0,0)
        major = adsk.core.Point3D.create(self._length,0,0)
        minor = adsk.core.Point3D.create(0,self._radius,0)

        ellipse = sketch.sketchCurves.sketchEllipses.add(center, major, minor)
        
        ellipse.majorAxisLine.deleteMe()
        ellipse.minorAxisLine.deleteMe()

        line1 = sketch.sketchCurves.sketchLines.addByTwoPoints(major, center)
        line2 = sketch.sketchCurves.sketchLines.addByTwoPoints(minor, center)
        
        negMinor = adsk.core.Point3D.create(0,-self._radius,0)
        ellipse.trim(negMinor)
    
    def drawSolidShoulder(self, sketch):
        center = adsk.core.Point3D.create(0,0,0)
        major = adsk.core.Point3D.create(self._length,0,0)
        minor = adsk.core.Point3D.create(0,self._radius,0)

        ellipse = sketch.sketchCurves.sketchEllipses.add(center, major, minor)
        
        ellipse.majorAxisLine.deleteMe()
        ellipse.minorAxisLine.deleteMe()

        line1 = sketch.sketchCurves.sketchLines.addByTwoPoints(major,
                    adsk.core.Point3D.create(-self._shoulderLength,0,0))
        line2 = sketch.sketchCurves.sketchLines.addByTwoPoints(line1.endSketchPoint,
                    adsk.core.Point3D.create(-self._shoulderLength,self._shoulderRadius,0))
        line3 = sketch.sketchCurves.sketchLines.addByTwoPoints(line2.endSketchPoint,
                    adsk.core.Point3D.create(0,self._shoulderRadius,0))
        line4 = sketch.sketchCurves.sketchLines.addByTwoPoints(line3.endSketchPoint,
                    minor)
        
        negMinor = adsk.core.Point3D.create(0,-self._radius,0)
        ellipse.trim(negMinor)
    
    def drawHollow(self, sketch):
        center = adsk.core.Point3D.create(0,0,0)
        major = adsk.core.Point3D.create(self._length,0,0)
        minor = adsk.core.Point3D.create(0,self._radius,0)

        ellipse = sketch.sketchCurves.sketchEllipses.add(center, major, minor)
        
        ellipse.majorAxisLine.deleteMe()
        ellipse.minorAxisLine.deleteMe()

        innerMajor = adsk.core.Point3D.create(self._length - self._thickness,0,0)
        innerMinor = adsk.core.Point3D.create(0,self._radius - self._thickness,0)

        innerEllipse = sketch.sketchCurves.sketchEllipses.add(center, innerMajor, innerMinor)
        
        innerEllipse.majorAxisLine.deleteMe()
        innerEllipse.minorAxisLine.deleteMe()

        line1 = sketch.sketchCurves.sketchLines.addByTwoPoints(major, innerMajor)
        line2 = sketch.sketchCurves.sketchLines.addByTwoPoints(minor, innerMinor)
        
        negInnerMinor = adsk.core.Point3D.create(0,-self._radius + self._thickness,0)
        innerEllipse.trim(negInnerMinor)
        
        negMinor = adsk.core.Point3D.create(0,-self._radius,0)
        ellipse.trim(negMinor)
    
    def drawHollowShoulder(self, sketch):
        center = adsk.core.Point3D.create(0,0,0)
        major = adsk.core.Point3D.create(self._length,0,0)
        minor = adsk.core.Point3D.create(0,self._radius,0)

        ellipse = sketch.sketchCurves.sketchEllipses.add(center, major, minor)
        
        ellipse.majorAxisLine.deleteMe()
        ellipse.minorAxisLine.deleteMe()

        innerMajor = adsk.core.Point3D.create(self._length - self._thickness,0,0)
        innerMinor = adsk.core.Point3D.create(self._thickness,
                    self.ellipseRadius(self._length, self._radius, self._thickness) - self._thickness,
                    0)

        innerEllipse = sketch.sketchCurves.sketchEllipses.add(center, innerMajor, innerMinor)
        
        innerEllipse.majorAxisLine.deleteMe()
        innerEllipse.minorAxisLine.deleteMe()

        line1 = sketch.sketchCurves.sketchLines.addByTwoPoints(major, innerMajor)
        
        # This line was moved up in the calculation sequence to prevent intersecting lines
        line0 = sketch.sketchCurves.sketchLines.addByTwoPoints(
                    adsk.core.Point3D.create(self._thickness,self._shoulderRadius - self._thickness,0),
                    innerMinor)
        shoulderInnerMinor = adsk.core.Point3D.create(0,
                    self._radius - self._thickness,
                    0)
        innerEllipse.trim(shoulderInnerMinor)
       
        line2 = sketch.sketchCurves.sketchLines.addByTwoPoints(minor,
                    adsk.core.Point3D.create(0,self._shoulderRadius,0))
                    
        # Now that the curve endpoints are defined, trim before drawing any potentially intersecting lines
        negMinor = adsk.core.Point3D.create(0,-self._radius,0)
        ellipse.trim(negMinor)

        line3 = sketch.sketchCurves.sketchLines.addByTwoPoints(line2.endSketchPoint,
                    adsk.core.Point3D.create(-self._shoulderLength,self._shoulderRadius,0))
        line4 = sketch.sketchCurves.sketchLines.addByTwoPoints(line3.endSketchPoint,
                    adsk.core.Point3D.create(-self._shoulderLength,self._shoulderRadius - self._thickness,0))
        line5 = sketch.sketchCurves.sketchLines.addByTwoPoints(line4.endSketchPoint,
                    adsk.core.Point3D.create(self._thickness,self._shoulderRadius - self._thickness,0))
