#
# Copyright 2019 David Carter <dcarter@davesrocketshop.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"), 
# to deal in the Software without restriction, including without limitation 
# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
# and/or sell copies of the Software, and to permit persons to whom the 
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in 
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
# IN THE SOFTWARE.

#Author-David Carter
#Description-Parameterized nose cone generation for rockets large and small

import adsk.core, adsk.fusion, adsk.cam, traceback
import math
from .NoseShapeHandler import NoseShapeHandler

    
class HaackShapeHandler(NoseShapeHandler):

    def theta(self, x, length):
        return  math.acos(1 - 2*x/length);

    def haack_y(self, x, length, radius, coefficient):
        theta = self.theta(x, length)
        return  radius * math.sqrt(theta - math.sin(2 * theta)/2
			+ coefficient * math.pow(math.sin(theta), 3)) / math.sqrt(math.pi);

    def haack_curve(self, length, radius, resolution, coefficient, min = 0):
        
        # Create an object collection for the points.
        points = adsk.core.ObjectCollection.create()
        
        for i in range(0, int(resolution)):

            x = float(i) * ((length - min) / float(resolution))
            y = self.haack_y(x, length, radius, coefficient)
            points.add(adsk.core.Point3D.create(length - x,y,0))

        points.add(adsk.core.Point3D.create(min,radius,0))
        return points
            
    def findHaackY(self, thickness, length, radius, coefficient):
        min = 0
        max = length
        x = 0

        # Do a binary search to see where f(x) = thickness, to 1 mm
        while (max - min) > 0.1:
            y = self.haack_y(length - x, length, radius, coefficient)
            if (y == thickness):
                return x
            if (y > thickness):
                min = x
            else:
                max = x
            x = (max - min) / 2 + min
        return x
    
    def drawSolid(self, sketch):
        outer_curve = self.haack_curve(self._length, self._radius, self._resolution, self._coefficient)

        major = adsk.core.Point3D.create(self._length,0,0)
        minor = adsk.core.Point3D.create(0,self._radius,0)
        center = adsk.core.Point3D.create(0,0,0)
        
        # Create the spline.
        haack = sketch.sketchCurves.sketchFittedSplines.add(outer_curve)

        line1 = sketch.sketchCurves.sketchLines.addByTwoPoints(major, center)
        line2 = sketch.sketchCurves.sketchLines.addByTwoPoints(minor, center)
    
    def drawSolidShoulder(self, sketch):
        outer_curve = self.haack_curve(self._length, self._radius, self._resolution, self._coefficient)

        major = adsk.core.Point3D.create(self._length,0,0)
        minor = adsk.core.Point3D.create(0,self._radius,0)
        center = adsk.core.Point3D.create(0,0,0)
        
        # Create the spline.
        haack = sketch.sketchCurves.sketchFittedSplines.add(outer_curve)

        line1 = sketch.sketchCurves.sketchLines.addByTwoPoints(major,
                    adsk.core.Point3D.create(-self._shoulderLength,0,0))
        line2 = sketch.sketchCurves.sketchLines.addByTwoPoints(line1.endSketchPoint,
                    adsk.core.Point3D.create(-self._shoulderLength,self._shoulderRadius,0))
        line3 = sketch.sketchCurves.sketchLines.addByTwoPoints(line2.endSketchPoint,
                    adsk.core.Point3D.create(0,self._shoulderRadius,0))
        line4 = sketch.sketchCurves.sketchLines.addByTwoPoints(line3.endSketchPoint,
                    minor)
    
    def drawHollow(self, sketch):
        outer_curve = self.haack_curve(self._length, self._radius, self._resolution, self._coefficient)

        major = adsk.core.Point3D.create(self._length,0,0)
        minor = adsk.core.Point3D.create(0,self._radius,0)

        # Find the point where the thickness matches the desired thickness, so we don't get too narrow at the tip
        x = self.findHaackY(self._thickness, self._length, self._radius, self._coefficient)
        innerMajor = adsk.core.Point3D.create(x,0,0)
        innerMinor = adsk.core.Point3D.create(0,self._radius - self._thickness,0)

        inner_curve = self.haack_curve(x, self._radius - self._thickness, self._resolution, self._coefficient)

        # Create the splines.
        haack = sketch.sketchCurves.sketchFittedSplines.add(outer_curve)
        innerHaack = sketch.sketchCurves.sketchFittedSplines.add(inner_curve)

        line1 = sketch.sketchCurves.sketchLines.addByTwoPoints(major, innerMajor)
        line2 = sketch.sketchCurves.sketchLines.addByTwoPoints(minor, innerMinor)
    
    def drawHollowShoulder(self, sketch):
        outer_curve = self.haack_curve(self._length, self._radius, self._resolution, self._coefficient)

        major = adsk.core.Point3D.create(self._length,0,0)
        minor = adsk.core.Point3D.create(0,self._radius,0)

        # Find the point where the thickness matches the desired thickness, so we don't get too narrow at the tip
        x = self.findHaackY(self._thickness, self._length, self._radius, self._coefficient)
        innerMajor = adsk.core.Point3D.create(x,0,0)
        innerMinor = adsk.core.Point3D.create(self._thickness,self._radius - self._thickness,0)

        inner_curve = self.haack_curve(x, self._radius - self._thickness, self._resolution, self._coefficient, self._thickness)

        # Create the splines.
        haack = sketch.sketchCurves.sketchFittedSplines.add(outer_curve)
        innerHaack = sketch.sketchCurves.sketchFittedSplines.add(inner_curve)

        line1 = sketch.sketchCurves.sketchLines.addByTwoPoints(major, innerMajor)
        line2 = sketch.sketchCurves.sketchLines.addByTwoPoints(minor,
                    adsk.core.Point3D.create(0,self._shoulderRadius,0))
        line3 = sketch.sketchCurves.sketchLines.addByTwoPoints(line2.endSketchPoint,
                    adsk.core.Point3D.create(-self._shoulderLength,self._shoulderRadius,0))
        line4 = sketch.sketchCurves.sketchLines.addByTwoPoints(line3.endSketchPoint,
                    adsk.core.Point3D.create(-self._shoulderLength,self._shoulderRadius - self._thickness,0))
        line5 = sketch.sketchCurves.sketchLines.addByTwoPoints(line4.endSketchPoint,
                    adsk.core.Point3D.create(self._thickness,self._shoulderRadius - self._thickness,0))
        line6 = sketch.sketchCurves.sketchLines.addByTwoPoints(line5.endSketchPoint,
                    innerMinor)
    
class VonKarmanShapeHandler(HaackShapeHandler):
    def __init__(self, inputs):
        super().__init__(inputs)
        self._coefficient = 0.0 # A Von Karman is a special case of a Haack
