#
# Copyright 2019 David Carter <dcarter@davesrocketshop.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"), 
# to deal in the Software without restriction, including without limitation 
# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
# and/or sell copies of the Software, and to permit persons to whom the 
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in 
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
# IN THE SOFTWARE.

#Author-David Carter
#Description-Parameterized nose cone generation for rockets large and small

import adsk.core, adsk.fusion, adsk.cam, traceback
from .ConeShape import ConeShapeHandler
from .EllipseShape import EllipseShapeHandler
from .OgiveShape import OgiveShapeHandler
from .HaackShape import HaackShapeHandler, VonKarmanShapeHandler

# Global list to keep all event handlers in scope.
# This is only needed with Python.
handlers = []

def run(context):
    ui = None
    try:
        app = adsk.core.Application.get()
        ui  = app.userInterface

        # Get the CommandDefinitions collection.
        cmdDefs = ui.commandDefinitions
        
        # Create a button command definition.
        buttonNoseCone = cmdDefs.addButtonDefinition('drsNoseConePythonAddIn', 
                                                   'Nose Cone Generator', 
                                                   'Create a nose cone',
                                                   './Resources/NoseCones') #'./Resources/Sample')
        
        # Connect to the command created event.
        noseConeCommandCreated = NoseConeCommandCreatedEventHandler()
        buttonNoseCone.commandCreated.add(noseConeCommandCreated)
        handlers.append(noseConeCommandCreated)
        
        # Get the ADD-INS panel in the model workspace. 
        #addInsPanel = ui.allToolbarPanels.itemById('SolidScriptsAddinsPanel')
        addInsPanel = ui.allToolbarPanels.itemById('SketchPanel')

        # Get the control for the Stop Sketch command.
        sketchStopCmd = addInsPanel.controls.itemById('SketchStop')
        
        # Add the button to the bottom of the panel.
        #buttonControl = addInsPanel.controls.addCommand(buttonNoseCone, 'drsNoseConePythonAddIn', sketchStopCmd.index)
        buttonControl = addInsPanel.controls.addCommand(buttonNoseCone, 'SketchStop')
        addInsPanel.controls.addSeparator('drsNoseConePythonAddInSeparator', 'drsNoseConePythonAddIn', False)
    except:
        if ui:
            ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))


# Event handler for the commandCreated event.
class NoseConeCommandCreatedEventHandler(adsk.core.CommandCreatedEventHandler):
    def __init__(self):
        super().__init__()
    def notify(self, args):
        # Verify that a sketch is active.
        app = adsk.core.Application.get()
        if app.activeEditObject.objectType != adsk.fusion.Sketch.classType():
            ui = app.userInterface
            ui.messageBox('A sketch must be active for this command.')
            return False
        
        eventArgs = adsk.core.CommandCreatedEventArgs.cast(args)
        
        # Get the command
        cmd = eventArgs.command

        # Get the CommandInputs collection to create new command inputs.            
        inputs = cmd.commandInputs

        # Inputs
        noseType = inputs.addDropDownCommandInput('noseType', 'Nose Type', adsk.core.DropDownStyles.TextListDropDownStyle);
        noseTypeItems = noseType.listItems
        noseTypeItems.add('cone', True, '')
        noseTypeItems.add('elliptical', False, '')
        noseTypeItems.add('ogive', False, '')
        noseTypeItems.add('Von Karman', False, '')
        noseTypeItems.add('Haack', False, '')
            
        length = inputs.addValueInput('length', 'Length (mm)', 
                                           'mm', adsk.core.ValueInput.createByString('60.0 mm'))
        radius = inputs.addValueInput('radius', 'Radius (mm)', 
                                           'mm', adsk.core.ValueInput.createByString('10.0 mm'))
        coefficient = inputs.addValueInput('coefficient', 'Coefficient', 
                                           '', adsk.core.ValueInput.createByReal(0.0))
        resolution = inputs.addValueInput('resolution', 'Resolution', 
                                           '', adsk.core.ValueInput.createByString('5'))
        coefficient.isVisible = False # Only visible for Haack
        resolution.isVisible = False # Not required for a cone
           
        # We're either solid, or have a thickness
        solid = inputs.addBoolValueInput('solid', 'Solid', 
                                               True, '', True)
        thickness = inputs.addValueInput('thickness', 'Thickness (mm)', 
                                           'mm', adsk.core.ValueInput.createByString('2.0 mm'))
        thickness.isVisible = False

        # Options for creating a shoulder
        shoulder = inputs.addBoolValueInput('shoulder', 'Shoulder', 
                                               True, '', False)
        shoulderLength = inputs.addValueInput('shoulderLength', 'Shoulder Length (mm)', 
                                           'mm', adsk.core.ValueInput.createByString('30.0 mm'))
        shoulderRadius = inputs.addValueInput('shoulderRadius', 'Shoulder Radius (mm)', 
                                           'mm', adsk.core.ValueInput.createByString('8.0 mm'))
        #shoulder.isEnabled = False # disabled for now
        shoulderLength.isVisible = False
        shoulderRadius.isVisible = False

        # Connect to the execute event.
        onExecute = NoseConeCommandExecuteHandler()
        cmd.execute.add(onExecute)
        handlers.append(onExecute)
        
        # Connect to the inputChanged event.
        onInputChanged = NoseConeCommandInputChangedHandler()
        cmd.inputChanged.add(onInputChanged)
        handlers.append(onInputChanged)
	
# Event handler for the inputChanged event.
class NoseConeCommandInputChangedHandler(adsk.core.InputChangedEventHandler):
    def __init__(self):
        super().__init__()
    def notify(self, args):
        eventArgs = adsk.core.InputChangedEventArgs.cast(args)
        
        changedInput = eventArgs.input
        inputs = eventArgs.firingEvent.sender.commandInputs

        if changedInput.id == 'noseType':
            noseType = inputs.itemById('noseType')
            coefficient = inputs.itemById('coefficient')
            resolution = inputs.itemById('resolution')
			
            # Change the visibility of the coefficient input.
            if noseType.selectedItem.name == 'Haack':
                coefficient.isVisible = True
            else:
                coefficient.isVisible = False
			
            # Change the visibility of the resolution input.
            if noseType.selectedItem.name in ['cone', 'elliptical']:
                resolution.isVisible = False
            else:
                resolution.isVisible = True
                
        elif changedInput.id == 'solid':
            thickness = inputs.itemById('thickness')
            if changedInput.value == True:
                thickness.isVisible = False
            else:
                thickness.isVisible = True
                
        elif changedInput.id == 'shoulder':
            shoulderLength = inputs.itemById('shoulderLength')
            shoulderRadius = inputs.itemById('shoulderRadius')
            if changedInput.value == True:
                shoulderLength.isVisible = True
                shoulderRadius.isVisible = True
            else:
                shoulderLength.isVisible = False
                shoulderRadius.isVisible = False

# Event handler for the execute event.
class NoseConeCommandExecuteHandler(adsk.core.CommandEventHandler):
    def __init__(self):
        super().__init__()
    def notify(self, args):
        eventArgs = adsk.core.CommandEventArgs.cast(args)

        # Code to react to the event.
#        app = adsk.core.Application.get()
#        ui  = app.userInterface
#        ui.messageBox('In command execute event handler.')

        # Get the values from the command inputs. 
        inputs = eventArgs.command.commandInputs
        noseType = inputs.itemById('noseType').selectedItem.name
        
        if noseType == 'cone':
            cone = ConeShapeHandler(inputs)
            cone.draw()
        elif noseType == 'elliptical':
            ellipse = EllipseShapeHandler(inputs)
            ellipse.draw()
        elif noseType == 'ogive':
            ogive = OgiveShapeHandler(inputs)
            ogive.draw()
        elif noseType == 'Von Karman':
            vonKarman = VonKarmanShapeHandler(inputs)
            vonKarman.draw()
        elif noseType == 'Haack':
            haack = HaackShapeHandler(inputs)
            haack.draw()


def stop(context):
    try:
        app = adsk.core.Application.get()
        ui  = app.userInterface
        
        # Clean up the UI.
        cmdDef = ui.commandDefinitions.itemById('drsNoseConePythonAddIn')
        if cmdDef:
            cmdDef.deleteMe()
            
        addinsPanel = ui.allToolbarPanels.itemById('SketchPanel')
        cntrl = addinsPanel.controls.itemById('drsNoseConePythonAddIn')
        if cntrl:
            cntrl.deleteMe()
        cntrl = addinsPanel.controls.itemById('drsNoseConePythonAddInSeparator')
        if cntrl:
            cntrl.deleteMe()
    except:
        if ui:
            ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))	
